@@ -34,26 +34,22 @@ struct opti_norm
 
 Vector add(Vector vec1, Vector vec2)
 {
-    return (Vector){
-        .x = vec1.x + vec2.x, .y = vec1.y + vec2.y, .z = vec1.z + vec2.z};
+    return (Vector){vec1.x + vec2.x, vec1.y + vec2.y, vec1.z + vec2.z};
 };
 
 Vector sub(Vector vec1, Vector vec2)
 {
-    return (Vector){
-        .x = vec1.x - vec2.x, .y = vec1.y - vec2.y, .z = vec1.z - vec2.z};
+    return (Vector){vec1.x - vec2.x, vec1.y - vec2.y, vec1.z - vec2.z};
 };
 
 Vector multiply(Vector vec1, Vector vec2)
 {
-    return (Vector){
-        .x = vec1.x * vec2.x, .y = vec1.y * vec2.y, .z = vec1.z * vec2.z};
+    return (Vector){vec1.x * vec2.x, vec1.y * vec2.y, vec1.z * vec2.z};
 };
 
 Vector divide(Vector vec1, Vector vec2)
 {
-    return (Vector){
-        .x = vec1.x / vec2.x, .y = vec1.y / vec2.y, .z = vec1.z / vec2.z};
+    return (Vector){vec1.x / vec2.x, vec1.y / vec2.y, vec1.z / vec2.z};
 };
 
 double dot(Vector vec1, Vector vec2)
@@ -69,23 +65,18 @@ double magnitude(Vector vec)
 struct opti_norm normalize(Vector vec)
 {
     struct opti_norm r;
-    r.vector = (Vector){
-        .x = 0, .y = 0, .z = 0};
+    r.vector = (Vector){0, 0, 0};
     r.magnitude = magnitude(vec);
 
     if (r.magnitude != 0)
-    {
-        r.vector = (Vector){
-            .x = vec.x / r.magnitude, .y = vec.y / r.magnitude, .z = vec.z / r.magnitude};
-    }
+        r.vector = (Vector){vec.x / r.magnitude, vec.y / r.magnitude, vec.z / r.magnitude};
 
     return r;
 };
 
 Vector flatten(Vector vec)
 {
-    return (Vector){
-        .x = vec.x, .y = vec.y, .z = 0};
+    return (Vector){vec.x, vec.y, 0};
 }
 
 double angle(Vector vec1, Vector vec2)
@@ -95,31 +86,22 @@ double angle(Vector vec1, Vector vec2)
 
 Vector cross(Vector vec1, Vector vec2)
 {
-    return (Vector){
-        .x = (vec1.y * vec2.z) - (vec1.z * vec2.y), .y = (vec1.z * vec2.x) - (vec1.x * vec2.z), .z = (vec1.x * vec2.y) - (vec1.y * vec2.x)};
+    return (Vector){(vec1.y * vec2.z) - (vec1.z * vec2.y), (vec1.z * vec2.x) - (vec1.x * vec2.z), (vec1.x * vec2.y) - (vec1.y * vec2.x)};
 };
 
 Vector double_to_vector(double num)
 {
-    return (Vector){
-        .x = num, .y = num, .z = num};
+    return (Vector){num, num, num};
 }
 
 Vector clamp(Vector vec, Vector start, Vector end)
 {
     Vector s = normalize(vec).vector;
-    _Bool right = dot(s, cross(end, (Vector){
-                                        .x = 0, .y = 0, .z = -1})) < 0;
-    _Bool left = dot(s, cross(start, (Vector){
-                                         .x = 0, .y = 0, .z = -1})) > 0;
+    _Bool right = dot(s, cross(end, (Vector){0, 0, -1})) < 0;
+    _Bool left = dot(s, cross(start, (Vector){0, 0, -1})) > 0;
 
-    if ((dot(end, cross(start, (Vector){
-                                   .x = 0, .y = 0, .z = -1})) > 0)
-            ? (right && left)
-            : (right || left))
-    {
+    if ((dot(end, cross(start, (Vector){0, 0, -1})) > 0) ? (right && left) : (right || left))
         return vec;
-    }
 
     if (dot(start, s) < dot(end, s))
         return end;
@@ -170,21 +152,14 @@ struct post_correction correct_for_posts(Vector ball_location, Vector left_targe
 
 _Bool in_field(Vector point, int radius)
 {
-    point = (Vector){.x = fabs(point.x), .y = fabs(point.y), .z = fabs(point.z)};
+    point = (Vector){fabs(point.x), fabs(point.y), fabs(point.z)};
     return !((point.x > 4080 - radius) || (point.y > 5900 - radius) || (point.x > 880 - radius && point.y > 5105 - radius) || (point.x > 2650 && point.y > -point.x + 8025 - radius));
 };
 
-_Bool fast_shot(Vector car_location, Vector ball_intercept, double time_remaining, double cap_, int speed)
-{
-    return time_remaining <= min(cap_, distance(car_location, ball_intercept) / speed);
-}
-
 double find_slope(Vector shot_vector, Vector car_to_target)
 {
     double d = dot(shot_vector, car_to_target);
-    double e = fabs(dot(cross(shot_vector, (Vector){
-                                               0, 0, 1}),
-                        car_to_target));
+    double e = fabs(dot(cross(shot_vector, (Vector){0, 0, 1}), car_to_target));
 
     if (e == 0)
         return 10 * copysign(1, d);
@@ -220,10 +195,10 @@ struct jump_shot parse_slice_for_jump_shot_with_target(double time_remaining, do
             Vector right_vector = normalize(sub(pst_crrctn.right, ball_location)).vector;
             r.best_shot_vector = clamp(direction, left_vector, right_vector);
 
-            if (in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1) && fast_shot(car_location, ball_location, time_remaining, cap_, 250))
+            if (in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1))
             {
                 double slope = find_slope(r.best_shot_vector, car_to_ball);
-                if ((forward_flag && ball_location.z <= 300 && slope > 0) || (backward_flag && ball_location.z < 250 && slope > 0.5 && time_remaining < cap_ / 2))
+                if ((forward_flag && slope > 0) || (backward_flag && slope > 0.5))
                     r.found = 1;
             }
         }
@@ -256,7 +231,80 @@ struct jump_shot parse_slice_for_jump_shot(double time_remaining, double best_sh
     if ((forward_flag || backward_flag) && in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1))
     {
         double slope = find_slope(r.best_shot_vector, car_to_ball);
-        if ((forward_flag && ball_location.z <= 300 && slope > 0) || (backward_flag && ball_location.z < 250 && slope > 0.5 && time_remaining < cap_ / 2))
+        if ((forward_flag && slope > 0) || (backward_flag && slope > 0.5))
+            r.found = 1;
+    }
+
+    return r;
+};
+
+struct jump_shot parse_slice_for_double_jump_with_target(double time_remaining, double best_shot_value, Vector ball_location, Vector car_location, Vector car_forward, int car_boost, Vector left_target, Vector right_target, double cap_)
+{
+    struct jump_shot r;
+    r.found = -1;
+
+    Vector car_to_ball = sub(ball_location, car_location);
+    struct opti_norm car_to_ball_norm = normalize(car_to_ball);
+    Vector direction = car_to_ball_norm.vector;
+    double distance = car_to_ball_norm.magnitude;
+
+    double forward_angle = angle(flatten(direction), flatten(car_forward));
+    double backward_angle = Py_MATH_PI - forward_angle;
+
+    double forward_time = time_remaining - (forward_angle * 0.318);
+    double backward_time = time_remaining - (backward_angle * 0.418);
+
+    _Bool forward_flag = forward_time > 0 && (distance / forward_time) < ((car_boost > distance / 100) ? 2275 : 1400);
+    _Bool backward_flag = backward_time > 0 && (distance * 1.05 / backward_time) < 1200;
+
+    r.best_shot_vector = direction;
+
+    if (forward_flag || backward_flag)
+    {
+        struct post_correction pst_crrctn = correct_for_posts(ball_location, left_target, right_target);
+        if (!pst_crrctn.swapped)
+        {
+            Vector left_vector = normalize(sub(pst_crrctn.left, ball_location)).vector;
+            Vector right_vector = normalize(sub(pst_crrctn.right, ball_location)).vector;
+            r.best_shot_vector = clamp(direction, left_vector, right_vector);
+
+            if (in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1))
+            {
+                double slope = find_slope(r.best_shot_vector, car_to_ball);
+                if ((forward_flag && slope > 0) || (backward_flag && slope > 0.5))
+                    r.found = 1;
+            }
+        }
+    }
+
+    return r;
+};
+
+struct jump_shot parse_slice_for_double_jump(double time_remaining, double best_shot_value, Vector ball_location, Vector car_location, Vector car_forward, int car_boost, double cap_)
+{
+    struct jump_shot r;
+    r.found = -1;
+
+    Vector car_to_ball = sub(ball_location, car_location);
+    struct opti_norm car_to_ball_norm = normalize(car_to_ball);
+    Vector direction = car_to_ball_norm.vector;
+    double distance = car_to_ball_norm.magnitude;
+
+    double forward_angle = angle(flatten(direction), flatten(car_forward));
+    double backward_angle = Py_MATH_PI - forward_angle;
+
+    double forward_time = time_remaining - (forward_angle * 0.318);
+    double backward_time = time_remaining - (backward_angle * 0.418);
+
+    _Bool forward_flag = forward_time > 0 && (distance / forward_time) < ((car_boost > distance / 100) ? 2275 : 1400);
+    _Bool backward_flag = backward_time > 0 && (distance * 1.05 / backward_time) < 1200;
+
+    r.best_shot_vector = direction;
+
+    if ((forward_flag || backward_flag) && in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1))
+    {
+        double slope = find_slope(r.best_shot_vector, car_to_ball);
+        if ((forward_flag && slope > 0) || (backward_flag && slope > 0.5))
             r.found = 1;
     }
 
@@ -325,10 +373,9 @@ struct aerial_shot parse_slice_for_aerial_shot_with_target(double time_remaining
         Vector right_vector = normalize(sub(pst_crrctn.right, ball_location)).vector;
         r.ball_intercept = sub(ball_location, multiply(clamp(direction, left_vector, right_vector), double_to_vector(best_shot_value)));
 
-        if (in_field(r.ball_intercept, 1) && fast_shot(me.location, ball_location, time_remaining, cap_, 250))
+        if (in_field(r.ball_intercept, 1))
         {
             double slope = find_slope(r.ball_intercept, car_to_ball);
-
             if (slope > 0 && is_viable(time_remaining, ceil(best_shot_value - 92.75) * 2, boost_accel, gravity, me, r.ball_intercept))
                 r.found = 1;
         }
@@ -348,7 +395,6 @@ struct aerial_shot parse_slice_for_aerial_shot(double time_remaining, double bes
     if (in_field(r.ball_intercept, 1))
     {
         double slope = find_slope(r.ball_intercept, car_to_ball);
-
         if (slope > 0 && is_viable(time_remaining, ceil(best_shot_value - 92.75) * 2, boost_accel, gravity, me, r.ball_intercept))
             r.found = 1;
     }
@@ -384,6 +430,34 @@ static PyObject *method_parse_slice_for_jump_shot(PyObject *self, PyObject *args
     return Py_BuildValue("{s:i,s:(ddd)}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z);
 };
 
+static PyObject *method_parse_slice_for_double_jump_with_target(PyObject *self, PyObject *args)
+{
+    int car_boost;
+    double time_remaining, best_shot_value, cap_;
+    Vector ball_location, car_location, car_forward, left_target, right_target;
+
+    if (!PyArg_ParseTuple(args, "dd(ddd)(ddd)(ddd)i(ddd)(ddd)d", &time_remaining, &best_shot_value, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &left_target.x, &left_target.y, &left_target.z, &right_target.x, &right_target.y, &right_target.z, &cap_))
+        return NULL;
+
+    struct jump_shot data_struct = parse_slice_for_double_jump_with_target(time_remaining, best_shot_value, ball_location, car_location, car_forward, car_boost, left_target, right_target, cap_);
+
+    return Py_BuildValue("{s:i,s:(ddd)}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z);
+};
+
+static PyObject *method_parse_slice_for_double_jump(PyObject *self, PyObject *args)
+{
+    int car_boost;
+    double time_remaining, best_shot_value, cap_;
+    Vector ball_location, car_location, car_forward;
+
+    if (!PyArg_ParseTuple(args, "dd(ddd)(ddd)(ddd)id", &time_remaining, &best_shot_value, &ball_location.x, &ball_location.y, &ball_location.z, &car_location.x, &car_location.y, &car_location.z, &car_forward.x, &car_forward.y, &car_forward.z, &car_boost, &cap_))
+        return NULL;
+
+    struct jump_shot data_struct = parse_slice_for_double_jump(time_remaining, best_shot_value, ball_location, car_location, car_forward, car_boost, cap_);
+
+    return Py_BuildValue("{s:i,s:(ddd)}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z);
+};
+
 static PyObject *method_parse_slice_for_aerial_shot_with_target(PyObject *self, PyObject *args)
 {
     struct car me;
@@ -415,6 +489,8 @@ static PyObject *method_parse_slice_for_aerial_shot(PyObject *self, PyObject *ar
 static PyMethodDef methods[] = {
     {"parse_slice_for_jump_shot_with_target", method_parse_slice_for_jump_shot_with_target, METH_VARARGS, "Parse slice for a jump shot with a target"},
     {"parse_slice_for_jump_shot", method_parse_slice_for_jump_shot, METH_VARARGS, "Parse slice for a jump shot"},
+    {"parse_slice_for_double_jump_with_target", method_parse_slice_for_double_jump_with_target, METH_VARARGS, "Parse slice for a double jump with a target"},
+    {"parse_slice_for_double_jump", method_parse_slice_for_double_jump, METH_VARARGS, "Parse slice for a double jump"},
     {"parse_slice_for_aerial_shot_with_target", method_parse_slice_for_aerial_shot_with_target, METH_VARARGS, "Parse slice for an aerial shot with a target"},
     {"parse_slice_for_aerial_shot", method_parse_slice_for_aerial_shot, METH_VARARGS, "Parse slice for an aerial shot"},
     {NULL, NULL, 0, NULL}};
